<?php

namespace Litepie\Plan\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litepie\Plan\Forms\PlanFeature as PlanFeatureForm;
use Litepie\Plan\Http\Requests\PlanFeatureRequest;
use Litepie\Plan\Interfaces\PlanFeatureRepositoryInterface;
use Litepie\Plan\Repositories\Eloquent\Filters\PlanFeatureResourceFilter;
use Litepie\Plan\Repositories\Eloquent\Presenters\PlanFeatureListPresenter;

/**
 * Resource controller class for plan_feature.
 */
class PlanFeatureResourceController extends BaseController
{

    /**
     * Initialize plan_feature resource controller.
     *
     *
     * @return null
     */
    public function __construct(PlanFeatureRepositoryInterface $plan_feature)
    {
        parent::__construct();
        $this->form = PlanFeatureForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litepie.plan.modules'), 'plan', guard_url('plan'));
        $this->repository = $plan_feature;
    }

    /**
     * Display a list of plan_feature.
     *
     * @return Response
     */
    public function index(PlanFeatureRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(PlanFeatureResourceFilter::class)
            ->setPresenter(PlanFeatureListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('plan::plan_feature.names'))
            ->view('plan::plan_feature.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display plan_feature.
     *
     * @param Request $request
     * @param Model   $plan_feature
     *
     * @return Response
     */
    public function show(PlanFeatureRequest $request, PlanFeatureRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('plan::plan_feature.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('plan::plan_feature.show')
            ->output();
    }

    /**
     * Show the form for creating a new plan_feature.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(PlanFeatureRequest $request, PlanFeatureRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('plan::plan_feature.name'))
            ->view('plan::plan_feature.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new plan_feature.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PlanFeatureRequest $request, PlanFeatureRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('plan::plan_feature.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('plan/plan_feature/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/plan/plan_feature'))
                ->redirect();
        }

    }

    /**
     * Show plan_feature for editing.
     *
     * @param Request $request
     * @param Model   $plan_feature
     *
     * @return Response
     */
    public function edit(PlanFeatureRequest $request, PlanFeatureRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('plan::plan_feature.name'))
            ->view('plan::plan_feature.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the plan_feature.
     *
     * @param Request $request
     * @param Model   $plan_feature
     *
     * @return Response
     */
    public function update(PlanFeatureRequest $request, PlanFeatureRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('plan::plan_feature.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('plan/plan_feature/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/plan_feature/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the plan_feature.
     *
     * @param Model   $plan_feature
     *
     * @return Response
     */
    public function destroy(PlanFeatureRequest $request, PlanFeatureRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::plan_feature.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('plan/plan_feature/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/plan_feature/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
