<?php

namespace Litepie\Plan\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litepie\Plan\Forms\Plan as PlanForm;
use Litepie\Plan\Http\Requests\PlanRequest;
use Litepie\Plan\Interfaces\PlanRepositoryInterface;
use Litepie\Plan\Repositories\Eloquent\Filters\PlanResourceFilter;
use Litepie\Plan\Repositories\Eloquent\Presenters\PlanListPresenter;

/**
 * Resource controller class for plan.
 */
class PlanResourceController extends BaseController
{

    /**
     * Initialize plan resource controller.
     *
     *
     * @return null
     */
    public function __construct(PlanRepositoryInterface $plan)
    {
        parent::__construct();
        $this->form = PlanForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litepie.plan.modules'), 'plan', guard_url('plan'));
        $this->repository = $plan;
    }

    /**
     * Display a list of plan.
     *
     * @return Response
     */
    public function index(PlanRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(PlanResourceFilter::class)
            ->setPresenter(PlanListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('plan::plan.names'))
            ->view('plan::plan.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display plan.
     *
     * @param Request $request
     * @param Model   $plan
     *
     * @return Response
     */
    public function show(PlanRequest $request, PlanRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('plan::plan.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('plan::plan.show')
            ->output();
    }

    /**
     * Show the form for creating a new plan.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(PlanRequest $request, PlanRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('plan::plan.name'))
            ->view('plan::plan.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new plan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PlanRequest $request, PlanRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('plan::plan.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('plan/plan/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/plan/plan'))
                ->redirect();
        }

    }

    /**
     * Show plan for editing.
     *
     * @param Request $request
     * @param Model   $plan
     *
     * @return Response
     */
    public function edit(PlanRequest $request, PlanRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('plan::plan.name'))
            ->view('plan::plan.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the plan.
     *
     * @param Request $request
     * @param Model   $plan
     *
     * @return Response
     */
    public function update(PlanRequest $request, PlanRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('plan::plan.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('plan/plan/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/plan/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the plan.
     *
     * @param Model   $plan
     *
     * @return Response
     */
    public function destroy(PlanRequest $request, PlanRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::plan.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('plan/plan/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/plan/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
